package com.bomostory.sceneeditmodule.screen.movie

import android.app.AlertDialog
import android.app.ProgressDialog
import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.DialogInterface
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.os.Environment
import android.os.Handler
import android.support.v4.app.DialogFragment
import android.support.v7.app.AppCompatActivity
import android.view.View
import android.widget.CompoundButton
import android.widget.SeekBar
import android.widget.Toast
import com.bomostory.sceneeditmodule.Config
import com.bomostory.sceneeditmodule.PdfMaker
import com.bomostory.sceneeditmodule.SuperMovieMaker
import com.bomostory.sceneeditmodule.basicdata.*
import com.bomostory.sceneeditmodule.screen.movie.music.MusicEditDialog
import com.bomostory.sceneeditmodule.screen.movie.music.MusicSelectDialog
import com.bomostory.sceneeditmodule.screen.movie.music.MusicSelectFragment
import com.bomostory.sceneeditmodule.share.ShareDialog
import com.bomostory.sceneeditmodule.utils.MoviePlayer
import com.bomostory.sceneeditmodule.utils.MusicPlayer
import com.bomostory.sceneeditmodule.screen.view.AudioTrackGroupView
import com.bomostory.sceneeditmodule.screen.view.MovieSelectView
import com.bomostory.sceneeditmodule.share.ExportPdfDialog
import com.example.exportmedia.MediaHelper
import com.example.tfat.myapplication.R
import com.google.gson.Gson
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import kotlinx.android.synthetic.main.activity_movie_edit.*
import java.io.File


class MovieEditActivity : AppCompatActivity(),
        DialogInterface.OnDismissListener,
        MusicSelectFragment.OnFragmentInteractionListener,
        MusicEditDialog.OnFragmentInteractionListener {

    override fun onMusicImportClick(music: Music) {
        val musics: ArrayList<Music> = viewModel.musicListLiveData.value as ArrayList<Music>
        musics.add(music)
        viewModel.musicListLiveData.value = musics
        viewModel.musicLiveData.value = music

        val dialogFragment = supportFragmentManager.findFragmentByTag(MUSIC_SELECT_DIALOG_TAG) as DialogFragment
        dialogFragment.dismiss()
    }

    override fun onMusicPreCastClick(music: Music) {
        musicPlayer.stop()
        musicPlayer.clear()
        musicPlayer.add(music)
        musicPlayer.play()
    }

    override fun onMusicPlayClick(music: Music, startPosition: Int) {
        musicPlayer.clear()
        musicPlayer.add(music)
        musicPlayer.playChangedListener = object : MusicPlayer.OnPlayChangedListener {
            override fun onPlayChanged(music: Music, position: Int) {
                runOnUiThread {
                    musicEditDialog?.currentPosition = position
                }
            }
        }
        musicPlayer.play(startPosition)
    }

    override fun onMusicVolumeChanged(volume: Float) {
        musicPlayer.setVolume(volume)
    }

    override fun onMusicPauseClick() {
        musicPlayer.pause()
    }

    override fun onMusicStop() {
        musicPlayer.stop()
    }

    override fun onMusicEditFinish(music: Music) {
        viewModel.exportEditAudio(music)
//        if (music.endTime - music.startTime > moviePlayer.totalPeriod) {
//            music.endTime = music.startTime + moviePlayer.totalPeriod
//        }
//
//        val builder = AudioEditor.Builder(mediaHelper)
//        builder.editAudioFile = File(music.path)
//        builder.startTime = TimeUtils.getEncodeTimeFormt(music.startTime)
//        builder.endTime = TimeUtils.getEncodeTimeFormt(music.endTime)
//        builder.volume = music.volume
//
//        val fileName = "test${System.currentTimeMillis()}.mp3"
//        val outputFile = File(Environment.getExternalStorageDirectory(), fileName)
//        val audioEditor = builder.build()
//        audioEditor.output(outputFile).subscribe({
//        }, {
//        })

    }

    override fun onDismiss(dialog: DialogInterface?) {
        musicPlayer.stop()
        musicPlayer.release()

        delayedHide(100)
    }

    private var musicEditDialog: MusicEditDialog? = null
    private val shareDialog = ShareDialog()

    private val mHideHandler = Handler()
    private val mHidePart2Runnable = Runnable {
        window.decorView.systemUiVisibility =
                View.SYSTEM_UI_FLAG_LOW_PROFILE or
                View.SYSTEM_UI_FLAG_FULLSCREEN or
                View.SYSTEM_UI_FLAG_LAYOUT_STABLE or
                View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY or
                View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION or
                View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
    }
    private val mShowPart2Runnable = Runnable {
        supportActionBar?.show()
    }
    private val mHideRunnable = Runnable {
        hide()
    }

    private val moviePlayer = MoviePlayer()

    private val musicPlayer = MusicPlayer()

    private val mediaHelper = MediaHelper()

    companion object {
        const val PROJECT_KEY = "project_key"
        const val MUSIC_SELECT_DIALOG_TAG = "music_select_dialog_tag"
        const val MUSIC_EDIT_DIALOG_TAG = "music_edit_dialog_tag"
        const val FPS: Int = 30
        private const val UI_ANIMATION_DELAY = 300
    }

    private lateinit var viewModel: MovieEditViewModel

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_movie_edit)

        mediaHelper.init(this)

        viewModel = ViewModelProviders.of(this).get(MovieEditViewModel::class.java)
        viewModel.mediaHelper = mediaHelper
        viewModel.project = Gson().fromJson<Project>(intent.extras.getString(PROJECT_KEY), Project::class.java)
        viewModel.storyLiveData.observe(this, Observer {
            movieEditView.story = it
            moviePlayer.story = it
        })
        viewModel.storyPeriodLiveData.observe(this, Observer {
            movieEditView.period = it ?: 0
            moviePlayer.totalPeriod = it ?: 0
        })
        viewModel.sceneLiveData.observe(this, Observer {
            movieView.scene = it
        })
        viewModel.musicListLiveData.observe(this, Observer {
            movieEditView.musics = it as ArrayList<Music>
            movieEditView.isAudioTrackGroupViewVisible = it.isNotEmpty()
            moviePlayer.musics = it
            moviePlayer.init()

        })
        viewModel.musicLiveData.observe(this, Observer {
            viewModel.musicListLiveData.value?.apply {
                movieEditView.currentAudioPosition = indexOf(it)
            }
        })

        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        movieEditView.onPlayClickListener = View.OnClickListener(this::playMovie)
        movieEditView.onPauseClickListener = View.OnClickListener(this::pauseMovie)
        movieEditView.onAddMusicClickListener = View.OnClickListener(this::addMusic)
        movieEditView.onEditMusicClickListener = View.OnClickListener(this::editMusic)
        movieEditView.onDeleteMusicClickListener = View.OnClickListener(this::deleteMusic)
        movieEditView.onSaveMovieClickListener = View.OnClickListener(this::onClickSaveAndShareBtn)
        movieEditView.onSeekBarChangeListener = object : SeekBar.OnSeekBarChangeListener {
            override fun onProgressChanged(seekBar: SeekBar?, progress: Int, fromUser: Boolean) {
                if (fromUser) {
                    moviePlayer.seekTo(progress)
                }
            }

            override fun onStartTrackingTouch(seekBar: SeekBar?) {
            }

            override fun onStopTrackingTouch(seekBar: SeekBar?) {
            }
        }
        movieEditView.onMovieSelectListener = object : MovieSelectView.OnMovieSelectListener {
            override fun onMoveSelect(scene: Scene) {
                viewModel.sceneLiveData.value = scene
            }
        }
        movieEditView.onAudioTrackSelectListener = object : AudioTrackGroupView.OnAudioTrackSelectListener {
            override fun onAudioTrackSelect(position: Int) {
                viewModel.musicListLiveData.value?.let {
                    viewModel.musicLiveData.value = it[position]
                }
            }
        }
        movieEditView.onAudioTrackScrollListener = object : AudioTrackGroupView.OnAudioTrackScrollListener {
            override fun onAudioTrackScroll(position: Int, scrollRatio: Double) {
                val musics: ArrayList<Music> = viewModel.musicListLiveData.value as ArrayList<Music>
                viewModel.sceneLiveData.value?.record?.period?.let {
                    musics[position].offsetTime = (it * scrollRatio).toLong()
                }
                viewModel.musicListLiveData.value = musics
            }
        }
        movieEditView.onLoopSwitchChangedListener = CompoundButton.OnCheckedChangeListener { _, isChecked ->
            val musics: ArrayList<Music> = viewModel.musicListLiveData.value as ArrayList<Music>
            for (music in musics) {
                music.isLoop = isChecked
            }
            viewModel.musicListLiveData.value = musics
        }

        moviePlayer.moviePlayListener = object : MoviePlayer.OnMoviePlayListener {
            override fun onMovieSceneUpdate(scene: Scene) {
                if (viewModel.sceneLiveData.value != scene) {
                    viewModel.sceneLiveData.value = scene
                }
            }

            override fun onMoviePlayComplete() {
                viewModel.project?.story?.let {
                    viewModel.sceneLiveData.value = it.scenes[0]
                }
                movieEditView.isBtnPanelEnable = true
            }

            override fun onMoviePlayViewUpdate(x: Int) {
                movieView.x = x
            }

            override fun onMoviePlayTimeUpdate(time: Long) {
                movieEditView.playTime = time
            }
        }


    }


    override fun onPostResume() {
        super.onPostResume()
        delayedHide(100)
    }

    private fun delayedHide(delayMillis: Int) {
        mHideHandler.removeCallbacks(mHideRunnable)
        mHideHandler.postDelayed(mHideRunnable, delayMillis.toLong())
    }

    private fun hide() {
        supportActionBar?.hide()
        mHideHandler.removeCallbacks(mShowPart2Runnable)
        mHideHandler.postDelayed(mHidePart2Runnable, UI_ANIMATION_DELAY.toLong())
    }

    private fun playMovie(view: View) {
        moviePlayer.play()
        movieEditView.isBtnPanelEnable = false
    }

    private fun pauseMovie(view: View) {
        moviePlayer.pause()
        movieEditView.isBtnPanelEnable = true
    }

    private fun addMusic(view: View) {
        viewModel.parseDeviceMusic(contentResolver)

        val musicSelectDialog = MusicSelectDialog.newInstance(Gson().toJson(viewModel.musicSource))
        musicSelectDialog.show(supportFragmentManager, MUSIC_SELECT_DIALOG_TAG)
    }

    private fun editMusic(view: View) {
        musicEditDialog = MusicEditDialog.newInstance(Gson().toJson(viewModel.musicLiveData.value))
        musicEditDialog?.show(supportFragmentManager, MUSIC_EDIT_DIALOG_TAG)
    }

    private fun deleteMusic(view: View) {
        val musics: ArrayList<Music> = viewModel.musicListLiveData.value as ArrayList<Music>
        musics.remove(viewModel.musicLiveData.value)
        viewModel.musicListLiveData.value = musics
    }

    private fun onClickSaveAndShareBtn(view: View) {
        shareDialog.apply {
            project = viewModel.project ?: return@apply
            onClickBomo = Runnable { saveMovie() }
            onClickExportPdf = Runnable { onClickExportPdf() }
        }.show(supportFragmentManager)
    }

    private fun onClickExportPdf() {
        ExportPdfDialog().also { exportPdfDialog ->
            exportPdfDialog.image1Path = viewModel.project?.scene1File?.path ?: ""
            exportPdfDialog.image2Path = viewModel.project?.scene2File?.path ?: ""
            exportPdfDialog.onClickPrint = Runnable {
                when (exportPdfDialog.type) {
                    ExportPdfDialog.Type.Standard -> {
                        val name = "standard_${viewModel.project?.name}_${System.currentTimeMillis()}.pdf"
                        if (!Config.PDF_FOLDER.exists()) {
                            Config.PDF_FOLDER.mkdirs()
                        }
                        val file = File(Config.PDF_FOLDER, name)
                        file.createNewFile()
                        val observable = PdfMaker.makeStandard(this@MovieEditActivity, viewModel.project!!, file)
                        Pair(file, observable)
                    }
                    ExportPdfDialog.Type.Booklet -> {
                        val name = "booklet_${viewModel.project?.name}_${System.currentTimeMillis()}.pdf"
                        if (!Config.PDF_FOLDER.exists()) {
                            Config.PDF_FOLDER.mkdirs()
                        }
                        val file = File(Config.PDF_FOLDER, name)
                        file.createNewFile()
                        val observable = PdfMaker.makeBooklet(this@MovieEditActivity, viewModel.project!!, file)
                        Pair(file, observable)
                    }
                }.apply {
                    exportPdfFile(second, Runnable {
                        exportPdfDialog.dismiss()
                    }, Runnable {
                        showPdfExportCompleteDialog(first)
                    })
                }
            }
        }.show(supportFragmentManager)
    }

    private fun saveMovie() {
        val scaledWidth = resources.getDimensionPixelSize(R.dimen.movie_width)
        val scaleHeight = resources.getDimensionPixelSize(R.dimen.movie_height)

        val pd = ProgressDialog(this).apply {
            val t = viewModel.storyPeriodLiveData.value ?: 0
            max = (t / 33).toInt()
            setMessage("progressing")
            setCancelable(false)
        }

        val fileName = "test${System.currentTimeMillis()}.mp4"
        val outputFile = File(Environment.getExternalStorageDirectory(), fileName)
        val project = viewModel.project ?: return

        SuperMovieMaker().make(
                outputFile,
                project,
                viewModel.musicListLiveData.value ?: ArrayList<Music>(),
                scaledWidth,
                scaleHeight,
                mediaHelper)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .doOnSubscribe {
                    pd.show()
                    delayedHide(100)
                }
                .doFinally { pd.dismiss() }
                .subscribe({
                    pd.setMessage(it)
                }, {
                    val msg = """
                    Error:
                    ${it.message}
                """.trimIndent()
                    AlertDialog.Builder(this).setMessage(msg).show()
                    delayedHide(100)
                }, {
                    AlertDialog.Builder(this)
                            .setMessage("Complete")
                            .setPositiveButton("Play") { _, _ ->
                                val uri = Uri.parse(outputFile.absolutePath)
                                val intent = Intent(Intent.ACTION_VIEW, uri).apply {
                                    setDataAndType(uri, "video/mp4")
                                }
                                if (intent.resolveActivity(packageManager) != null) {
                                    Intent.createChooser(intent, "").apply {
                                        startActivity(this)
                                    }
                                } else {
                                    Toast.makeText(this, "No app can play video", Toast.LENGTH_SHORT).show()
                                }
                            }.show()
                    delayedHide(100)
                })
    }

    private fun exportPdfFile(observable: Observable<Float>, onFinally: Runnable, onComplete: Runnable) {
        val pd = ProgressDialog(this@MovieEditActivity).apply {
            setCancelable(false)
        }
        observable.subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .doOnSubscribe { pd.show() }
                .doFinally {
                    pd.dismiss()
                    onFinally.run()
                }
                .subscribe({
                    val progress = (it * 100).toInt()
                    val msg = if (progress >= 100) {
                        "processing ..."
                    } else {
                        "$progress%"
                    }
                    pd.setMessage(msg)
                }, {
                }, {
                    onComplete.run()
                })
    }

    private fun showPdfExportCompleteDialog(file: File) {
        AlertDialog.Builder(this@MovieEditActivity)
                .setMessage("Complete")
                .setPositiveButton("View") { _, _ ->
                    val uri = Uri.parse(file.absolutePath)
                    val intent = Intent(Intent.ACTION_VIEW, uri).apply {
                        setDataAndType(uri, "application/pdf")
                    }
                    if (intent.resolveActivity(packageManager) != null) {
                        Intent.createChooser(intent, "").apply {
                            startActivity(this)
                        }
                    } else {
                        Toast.makeText(this@MovieEditActivity, "No app can view pdf", Toast.LENGTH_SHORT).show()
                    }
                }.show()
    }
}